# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from datetime import datetime
from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region,
    ScwFile,
    unmarshal_ScwFile,
)
from scaleway_core.utils import (
    WaitForOptions,
    fetch_all_pages,
    random_name,
    validate_path_param,
    wait_for_resource,
)
from .types import (
    ListDatabaseBackupsRequestOrderBy,
    ListDatabasesRequestOrderBy,
    ListInstanceLogsRequestOrderBy,
    ListInstancesRequestOrderBy,
    ListPrivilegesRequestOrderBy,
    ListSnapshotsRequestOrderBy,
    ListUsersRequestOrderBy,
    Permission,
    VolumeType,
    ACLRule,
    ACLRuleRequest,
    AddInstanceACLRulesResponse,
    AddInstanceSettingsResponse,
    Database,
    DatabaseBackup,
    DatabaseEngine,
    DeleteInstanceACLRulesResponse,
    DeleteInstanceSettingsResponse,
    Endpoint,
    EndpointSpec,
    Instance,
    InstanceLog,
    InstanceMetrics,
    InstanceSetting,
    ListDatabaseBackupsResponse,
    ListDatabaseEnginesResponse,
    ListDatabasesResponse,
    ListInstanceACLRulesResponse,
    ListInstanceLogsDetailsResponse,
    ListInstanceLogsResponse,
    ListInstancesResponse,
    ListNodeTypesResponse,
    ListPrivilegesResponse,
    ListSnapshotsResponse,
    ListUsersResponse,
    LogsPolicy,
    NodeType,
    PrepareInstanceLogsResponse,
    Privilege,
    ReadReplica,
    ReadReplicaEndpointSpec,
    SetInstanceACLRulesResponse,
    SetInstanceSettingsResponse,
    Snapshot,
    User,
    CreateDatabaseBackupRequest,
    UpdateDatabaseBackupRequest,
    RestoreDatabaseBackupRequest,
    UpgradeInstanceRequest,
    CreateInstanceRequest,
    UpdateInstanceRequest,
    CloneInstanceRequest,
    CreateReadReplicaRequest,
    CreateReadReplicaEndpointRequest,
    PrepareInstanceLogsRequest,
    PurgeInstanceLogsRequest,
    AddInstanceSettingsRequest,
    DeleteInstanceSettingsRequest,
    SetInstanceSettingsRequest,
    AddInstanceACLRulesRequest,
    SetInstanceACLRulesRequest,
    DeleteInstanceACLRulesRequest,
    CreateUserRequest,
    UpdateUserRequest,
    CreateDatabaseRequest,
    SetPrivilegeRequest,
    CreateSnapshotRequest,
    UpdateSnapshotRequest,
    CreateInstanceFromSnapshotRequest,
    CreateEndpointRequest,
    MigrateEndpointRequest,
)
from .content import (
    DATABASE_BACKUP_TRANSIENT_STATUSES,
    INSTANCE_LOG_TRANSIENT_STATUSES,
    INSTANCE_TRANSIENT_STATUSES,
    READ_REPLICA_TRANSIENT_STATUSES,
    SNAPSHOT_TRANSIENT_STATUSES,
)
from .marshalling import (
    marshal_AddInstanceACLRulesRequest,
    marshal_AddInstanceSettingsRequest,
    marshal_CloneInstanceRequest,
    marshal_CreateDatabaseBackupRequest,
    marshal_CreateDatabaseRequest,
    marshal_CreateEndpointRequest,
    marshal_CreateInstanceFromSnapshotRequest,
    marshal_CreateInstanceRequest,
    marshal_CreateReadReplicaEndpointRequest,
    marshal_CreateReadReplicaRequest,
    marshal_CreateSnapshotRequest,
    marshal_CreateUserRequest,
    marshal_DeleteInstanceACLRulesRequest,
    marshal_DeleteInstanceSettingsRequest,
    marshal_MigrateEndpointRequest,
    marshal_PrepareInstanceLogsRequest,
    marshal_PurgeInstanceLogsRequest,
    marshal_RestoreDatabaseBackupRequest,
    marshal_SetInstanceACLRulesRequest,
    marshal_SetInstanceSettingsRequest,
    marshal_SetPrivilegeRequest,
    marshal_UpdateDatabaseBackupRequest,
    marshal_UpdateInstanceRequest,
    marshal_UpdateSnapshotRequest,
    marshal_UpdateUserRequest,
    marshal_UpgradeInstanceRequest,
    unmarshal_Endpoint,
    unmarshal_ReadReplica,
    unmarshal_Database,
    unmarshal_DatabaseBackup,
    unmarshal_Instance,
    unmarshal_InstanceLog,
    unmarshal_Privilege,
    unmarshal_Snapshot,
    unmarshal_User,
    unmarshal_AddInstanceACLRulesResponse,
    unmarshal_AddInstanceSettingsResponse,
    unmarshal_DeleteInstanceACLRulesResponse,
    unmarshal_DeleteInstanceSettingsResponse,
    unmarshal_InstanceMetrics,
    unmarshal_ListDatabaseBackupsResponse,
    unmarshal_ListDatabaseEnginesResponse,
    unmarshal_ListDatabasesResponse,
    unmarshal_ListInstanceACLRulesResponse,
    unmarshal_ListInstanceLogsDetailsResponse,
    unmarshal_ListInstanceLogsResponse,
    unmarshal_ListInstancesResponse,
    unmarshal_ListNodeTypesResponse,
    unmarshal_ListPrivilegesResponse,
    unmarshal_ListSnapshotsResponse,
    unmarshal_ListUsersResponse,
    unmarshal_PrepareInstanceLogsResponse,
    unmarshal_SetInstanceACLRulesResponse,
    unmarshal_SetInstanceSettingsResponse,
)


class RdbV1API(API):
    """
    Managed Database for PostgreSQL and MySQL API.

    Managed Database for PostgreSQL and MySQL API.
    """

    def list_database_engines(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        version: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListDatabaseEnginesResponse:
        """
        List available database engines.
        List the PostgreSQL and MySQL database engines available at Scaleway.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the database engine.
        :param version: Version of the database engine.
        :param page:
        :param page_size:
        :return: :class:`ListDatabaseEnginesResponse <ListDatabaseEnginesResponse>`

        Usage:
        ::

            result = api.list_database_engines()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/database-engines",
            params={
                "name": name,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "version": version,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListDatabaseEnginesResponse(res.json())

    def list_database_engines_all(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        version: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[DatabaseEngine]:
        """
        List available database engines.
        List the PostgreSQL and MySQL database engines available at Scaleway.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the database engine.
        :param version: Version of the database engine.
        :param page:
        :param page_size:
        :return: :class:`List[ListDatabaseEnginesResponse] <List[ListDatabaseEnginesResponse]>`

        Usage:
        ::

            result = api.list_database_engines_all()
        """

        return fetch_all_pages(
            type=ListDatabaseEnginesResponse,
            key="engines",
            fetcher=self.list_database_engines,
            args={
                "region": region,
                "name": name,
                "version": version,
                "page": page,
                "page_size": page_size,
            },
        )

    def list_node_types(
        self,
        *,
        include_disabled_types: bool,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListNodeTypesResponse:
        """
        List available node types.
        List all available node types. By default, the node types returned in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param include_disabled_types: Defines whether or not to include disabled types.
        :param page:
        :param page_size:
        :return: :class:`ListNodeTypesResponse <ListNodeTypesResponse>`

        Usage:
        ::

            result = api.list_node_types(include_disabled_types=True)
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/node-types",
            params={
                "include_disabled_types": include_disabled_types,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListNodeTypesResponse(res.json())

    def list_node_types_all(
        self,
        *,
        include_disabled_types: bool,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[NodeType]:
        """
        List available node types.
        List all available node types. By default, the node types returned in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param include_disabled_types: Defines whether or not to include disabled types.
        :param page:
        :param page_size:
        :return: :class:`List[ListNodeTypesResponse] <List[ListNodeTypesResponse]>`

        Usage:
        ::

            result = api.list_node_types_all(include_disabled_types=True)
        """

        return fetch_all_pages(
            type=ListNodeTypesResponse,
            key="node_types",
            fetcher=self.list_node_types,
            args={
                "include_disabled_types": include_disabled_types,
                "region": region,
                "page": page,
                "page_size": page_size,
            },
        )

    def list_database_backups(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        order_by: ListDatabaseBackupsRequestOrderBy = ListDatabaseBackupsRequestOrderBy.CREATED_AT_ASC,
        instance_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListDatabaseBackupsResponse:
        """
        List database backups.
        List all backups in a specified region, for a given Scaleway Organization or Scaleway Project. By default, the backups listed are ordered by creation date in ascending order. This can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the database backups.
        :param order_by: Criteria to use when ordering database backups listing.
        :param instance_id: UUID of the Database Instance.
        :param organization_id: Organization ID of the Organization the database backups belong to.
        :param project_id: Project ID of the Project the database backups belong to.
        :param page:
        :param page_size:
        :return: :class:`ListDatabaseBackupsResponse <ListDatabaseBackupsResponse>`

        Usage:
        ::

            result = api.list_database_backups()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/backups",
            params={
                "instance_id": instance_id,
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListDatabaseBackupsResponse(res.json())

    def list_database_backups_all(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        order_by: Optional[ListDatabaseBackupsRequestOrderBy] = None,
        instance_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[DatabaseBackup]:
        """
        List database backups.
        List all backups in a specified region, for a given Scaleway Organization or Scaleway Project. By default, the backups listed are ordered by creation date in ascending order. This can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the database backups.
        :param order_by: Criteria to use when ordering database backups listing.
        :param instance_id: UUID of the Database Instance.
        :param organization_id: Organization ID of the Organization the database backups belong to.
        :param project_id: Project ID of the Project the database backups belong to.
        :param page:
        :param page_size:
        :return: :class:`List[ListDatabaseBackupsResponse] <List[ListDatabaseBackupsResponse]>`

        Usage:
        ::

            result = api.list_database_backups_all()
        """

        return fetch_all_pages(
            type=ListDatabaseBackupsResponse,
            key="database_backups",
            fetcher=self.list_database_backups,
            args={
                "region": region,
                "name": name,
                "order_by": order_by,
                "instance_id": instance_id,
                "organization_id": organization_id,
                "project_id": project_id,
                "page": page,
                "page_size": page_size,
            },
        )

    def create_database_backup(
        self,
        *,
        instance_id: str,
        database_name: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        expires_at: Optional[datetime] = None,
    ) -> DatabaseBackup:
        """
        Create a database backup.
        Create a new backup. You must set the `instance_id`, `database_name`, `name` and `expires_at` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param database_name: Name of the database you want to back up.
        :param name: Name of the backup.
        :param expires_at: Expiration date (must follow the ISO 8601 format).
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.create_database_backup(
                instance_id="example",
                database_name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/backups",
            body=marshal_CreateDatabaseBackupRequest(
                CreateDatabaseBackupRequest(
                    instance_id=instance_id,
                    database_name=database_name,
                    region=region,
                    name=name or random_name(prefix="bkp"),
                    expires_at=expires_at,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DatabaseBackup(res.json())

    def get_database_backup(
        self,
        *,
        database_backup_id: str,
        region: Optional[Region] = None,
    ) -> DatabaseBackup:
        """
        Get a database backup.
        Retrieve information about a given backup, specified by its database backup ID and region. Full details about the backup, like size, URL and expiration date, are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param database_backup_id: UUID of the database backup.
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.get_database_backup(database_backup_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_database_backup_id = validate_path_param(
            "database_backup_id", database_backup_id
        )

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/backups/{param_database_backup_id}",
        )

        self._throw_on_error(res)
        return unmarshal_DatabaseBackup(res.json())

    def wait_for_database_backup(
        self,
        *,
        database_backup_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[DatabaseBackup, bool]] = None,
    ) -> DatabaseBackup:
        """
        Waits for :class:`DatabaseBackup <DatabaseBackup>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param database_backup_id: UUID of the database backup.
        :param options: The options for the waiter
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.wait_for_database_backup(database_backup_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = (
                lambda res: res.status not in DATABASE_BACKUP_TRANSIENT_STATUSES
            )

        return wait_for_resource(
            fetcher=self.get_database_backup,
            options=options,
            args={
                "database_backup_id": database_backup_id,
                "region": region,
            },
        )

    def update_database_backup(
        self,
        *,
        database_backup_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        expires_at: Optional[datetime] = None,
    ) -> DatabaseBackup:
        """
        Update a database backup.
        Update the parameters of a backup, including name and expiration date.
        :param region: Region to target. If none is passed will use default region from the config.
        :param database_backup_id: UUID of the database backup to update.
        :param name: Name of the Database Backup.
        :param expires_at: Expiration date (must follow the ISO 8601 format).
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.update_database_backup(database_backup_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_database_backup_id = validate_path_param(
            "database_backup_id", database_backup_id
        )

        res = self._request(
            "PATCH",
            f"/rdb/v1/regions/{param_region}/backups/{param_database_backup_id}",
            body=marshal_UpdateDatabaseBackupRequest(
                UpdateDatabaseBackupRequest(
                    database_backup_id=database_backup_id,
                    region=region,
                    name=name,
                    expires_at=expires_at,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DatabaseBackup(res.json())

    def delete_database_backup(
        self,
        *,
        database_backup_id: str,
        region: Optional[Region] = None,
    ) -> DatabaseBackup:
        """
        Delete a database backup.
        Delete a backup, specified by its database backup ID and region. Deleting a backup is permanent, and cannot be undone.
        :param region: Region to target. If none is passed will use default region from the config.
        :param database_backup_id: UUID of the database backup to delete.
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.delete_database_backup(database_backup_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_database_backup_id = validate_path_param(
            "database_backup_id", database_backup_id
        )

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/backups/{param_database_backup_id}",
        )

        self._throw_on_error(res)
        return unmarshal_DatabaseBackup(res.json())

    def restore_database_backup(
        self,
        *,
        database_backup_id: str,
        instance_id: str,
        region: Optional[Region] = None,
        database_name: Optional[str] = None,
    ) -> DatabaseBackup:
        """
        Restore a database backup.
        Launch the process of restoring database backup. You must specify the `instance_id` of the Database Instance of destination, where the backup will be restored. Note that large database backups can take up to several hours to restore.
        :param region: Region to target. If none is passed will use default region from the config.
        :param database_backup_id: Backup of a logical database.
        :param database_name: Defines the destination database to restore into a specified database (the default destination is set to the origin database of the backup).
        :param instance_id: Defines the Database Instance where the backup has to be restored.
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.restore_database_backup(
                database_backup_id="example",
                instance_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_database_backup_id = validate_path_param(
            "database_backup_id", database_backup_id
        )

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/backups/{param_database_backup_id}/restore",
            body=marshal_RestoreDatabaseBackupRequest(
                RestoreDatabaseBackupRequest(
                    database_backup_id=database_backup_id,
                    instance_id=instance_id,
                    region=region,
                    database_name=database_name,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DatabaseBackup(res.json())

    def export_database_backup(
        self,
        *,
        database_backup_id: str,
        region: Optional[Region] = None,
    ) -> DatabaseBackup:
        """
        Export a database backup.
        Export a backup, specified by the `database_backup_id` and the `region` parameters. The download URL is returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param database_backup_id: UUID of the database backup you want to export.
        :return: :class:`DatabaseBackup <DatabaseBackup>`

        Usage:
        ::

            result = api.export_database_backup(database_backup_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_database_backup_id = validate_path_param(
            "database_backup_id", database_backup_id
        )

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/backups/{param_database_backup_id}/export",
        )

        self._throw_on_error(res)
        return unmarshal_DatabaseBackup(res.json())

    def upgrade_instance(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        node_type: Optional[str] = None,
        enable_ha: Optional[bool] = None,
        volume_size: Optional[int] = None,
        volume_type: Optional[VolumeType] = None,
        upgradable_version_id: Optional[str] = None,
    ) -> Instance:
        """
        Upgrade a Database Instance.
        Upgrade your current Database Instance specifications like node type, high availability, volume, or the database engine version. Note that upon upgrade the `enable_ha` parameter can only be set to `true`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to upgrade.
        :param node_type: Node type of the Database Instance you want to upgrade to.

        One-of ('upgrade_target'): at most one of 'node_type', 'enable_ha', 'volume_size', 'volume_type', 'upgradable_version_id' could be set.
        :param enable_ha: Defines whether or not high availability should be enabled on the Database Instance.

        One-of ('upgrade_target'): at most one of 'node_type', 'enable_ha', 'volume_size', 'volume_type', 'upgradable_version_id' could be set.
        :param volume_size: Increase your block storage volume size.

        One-of ('upgrade_target'): at most one of 'node_type', 'enable_ha', 'volume_size', 'volume_type', 'upgradable_version_id' could be set.
        :param volume_type: Change your Database Instance storage type.

        One-of ('upgrade_target'): at most one of 'node_type', 'enable_ha', 'volume_size', 'volume_type', 'upgradable_version_id' could be set.
        :param upgradable_version_id: Update your database engine to a newer version.
        This will create a new Database Instance with same specifications as the current one and perform a Database Engine upgrade.

        One-of ('upgrade_target'): at most one of 'node_type', 'enable_ha', 'volume_size', 'volume_type', 'upgradable_version_id' could be set.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.upgrade_instance(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/upgrade",
            body=marshal_UpgradeInstanceRequest(
                UpgradeInstanceRequest(
                    instance_id=instance_id,
                    region=region,
                    node_type=node_type,
                    enable_ha=enable_ha,
                    volume_size=volume_size,
                    volume_type=volume_type,
                    upgradable_version_id=upgradable_version_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def list_instances(
        self,
        *,
        region: Optional[Region] = None,
        tags: Optional[List[str]] = None,
        name: Optional[str] = None,
        order_by: ListInstancesRequestOrderBy = ListInstancesRequestOrderBy.CREATED_AT_ASC,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListInstancesResponse:
        """
        List Database Instances.
        List all Database Instances in the specified region, for a given Scaleway Organization or Scaleway Project. By default, the Database Instances returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field. You can define additional parameters for your query, such as `tags` and `name`. For the `name` parameter, the value you include will be checked against the whole name string to see if it includes the string you put in the parameter.
        :param region: Region to target. If none is passed will use default region from the config.
        :param tags: List Database Instances that have a given tag.
        :param name: Lists Database Instances that match a name pattern.
        :param order_by: Criteria to use when ordering Database Instance listings.
        :param organization_id: Please use project_id instead.
        :param project_id: Project ID to list the Database Instance of.
        :param page:
        :param page_size:
        :return: :class:`ListInstancesResponse <ListInstancesResponse>`

        Usage:
        ::

            result = api.list_instances()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "tags": tags,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListInstancesResponse(res.json())

    def list_instances_all(
        self,
        *,
        region: Optional[Region] = None,
        tags: Optional[List[str]] = None,
        name: Optional[str] = None,
        order_by: Optional[ListInstancesRequestOrderBy] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[Instance]:
        """
        List Database Instances.
        List all Database Instances in the specified region, for a given Scaleway Organization or Scaleway Project. By default, the Database Instances returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field. You can define additional parameters for your query, such as `tags` and `name`. For the `name` parameter, the value you include will be checked against the whole name string to see if it includes the string you put in the parameter.
        :param region: Region to target. If none is passed will use default region from the config.
        :param tags: List Database Instances that have a given tag.
        :param name: Lists Database Instances that match a name pattern.
        :param order_by: Criteria to use when ordering Database Instance listings.
        :param organization_id: Please use project_id instead.
        :param project_id: Project ID to list the Database Instance of.
        :param page:
        :param page_size:
        :return: :class:`List[ListInstancesResponse] <List[ListInstancesResponse]>`

        Usage:
        ::

            result = api.list_instances_all()
        """

        return fetch_all_pages(
            type=ListInstancesResponse,
            key="instances",
            fetcher=self.list_instances,
            args={
                "region": region,
                "tags": tags,
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id,
                "project_id": project_id,
                "page": page,
                "page_size": page_size,
            },
        )

    def get_instance(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> Instance:
        """
        Get a Database Instance.
        Retrieve information about a given Database Instance, specified by the `region` and `instance_id` parameters. Its full details, including name, status, IP address and port, are returned in the response object.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.get_instance(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def wait_for_instance(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Instance, bool]] = None,
    ) -> Instance:
        """
        Waits for :class:`Instance <Instance>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param options: The options for the waiter
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.wait_for_instance(instance_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in INSTANCE_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_instance,
            options=options,
            args={
                "instance_id": instance_id,
                "region": region,
            },
        )

    def create_instance(
        self,
        *,
        engine: str,
        user_name: str,
        password: str,
        node_type: str,
        is_ha_cluster: bool,
        disable_backup: bool,
        volume_type: VolumeType,
        volume_size: int,
        backup_same_region: bool,
        region: Optional[Region] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
        init_settings: Optional[List[InstanceSetting]] = None,
        init_endpoints: Optional[List[EndpointSpec]] = None,
    ) -> Instance:
        """
        Create a Database Instance.
        Create a new Database Instance. You must set the `engine`, `user_name`, `password` and `node_type` parameters. Optionally, you can specify the volume type and size.
        :param region: Region to target. If none is passed will use default region from the config.
        :param organization_id: Please use project_id instead.

        One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
        :param project_id: The Project ID on which the Database Instance will be created.

        One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
        :param name: Name of the Database Instance.
        :param engine: Database engine of the Database Instance (PostgreSQL, MySQL, ...).
        :param user_name: Username created when the Database Instance is created.
        :param password: Password of the user.
        :param node_type: Type of node to use for the Database Instance.
        :param is_ha_cluster: Defines whether or not High-Availability is enabled.
        :param disable_backup: Defines whether or not backups are disabled.
        :param tags: Tags to apply to the Database Instance.
        :param init_settings: List of engine settings to be set upon Database Instance initialization.
        :param volume_type: Type of volume where data is stored (lssd, bssd, ...).
        :param volume_size: Volume size when volume_type is not lssd.
        :param init_endpoints: One or multiple EndpointSpec used to expose your Database Instance. A load_balancer public endpoint is systematically created.
        :param backup_same_region: Defines whether to or not to store logical backups in the same region as the Database Instance.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.create_instance(
                engine="example",
                user_name="example",
                password="example",
                node_type="example",
                is_ha_cluster=True,
                disable_backup=True,
                volume_type=lssd,
                volume_size=1,
                backup_same_region=True,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances",
            body=marshal_CreateInstanceRequest(
                CreateInstanceRequest(
                    engine=engine,
                    user_name=user_name,
                    password=password,
                    node_type=node_type,
                    is_ha_cluster=is_ha_cluster,
                    disable_backup=disable_backup,
                    volume_type=volume_type,
                    volume_size=volume_size,
                    backup_same_region=backup_same_region,
                    region=region,
                    organization_id=organization_id,
                    project_id=project_id,
                    name=name or random_name(prefix="ins"),
                    tags=tags,
                    init_settings=init_settings,
                    init_endpoints=init_endpoints,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def update_instance(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        backup_schedule_frequency: Optional[int] = None,
        backup_schedule_retention: Optional[int] = None,
        is_backup_schedule_disabled: Optional[bool] = None,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
        logs_policy: Optional[LogsPolicy] = None,
        backup_same_region: Optional[bool] = None,
    ) -> Instance:
        """
        Update a Database Instance.
        Update the parameters of a Database Instance, including name, tags and backup schedule details.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance to update.
        :param backup_schedule_frequency: In hours.
        :param backup_schedule_retention: In days.
        :param is_backup_schedule_disabled: Defines whether or not the backup schedule is disabled.
        :param name: Name of the Database Instance.
        :param tags: Tags of a Database Instance.
        :param logs_policy: Logs policy of the Database Instance.
        :param backup_same_region: Store logical backups in the same region as the Database Instance.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.update_instance(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "PATCH",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}",
            body=marshal_UpdateInstanceRequest(
                UpdateInstanceRequest(
                    instance_id=instance_id,
                    region=region,
                    backup_schedule_frequency=backup_schedule_frequency,
                    backup_schedule_retention=backup_schedule_retention,
                    is_backup_schedule_disabled=is_backup_schedule_disabled,
                    name=name,
                    tags=tags,
                    logs_policy=logs_policy,
                    backup_same_region=backup_same_region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def delete_instance(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> Instance:
        """
        Delete a Database Instance.
        Delete a given Database Instance, specified by the `region` and `instance_id` parameters. Deleting a Database Instance is permanent, and cannot be undone. Note that upon deletion all your data will be lost.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance to delete.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.delete_instance(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def clone_instance(
        self,
        *,
        instance_id: str,
        name: str,
        region: Optional[Region] = None,
        node_type: Optional[str] = None,
    ) -> Instance:
        """
        Clone a Database Instance.
        Clone a given Database Instance, specified by the `region` and `instance_id` parameters. The clone feature allows you to create a new Database Instance from an existing one. The clone includes all existing databases, users and permissions. You can create a clone on a Database Instance bigger than your current one.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to clone.
        :param name: Name of the Database Instance clone.
        :param node_type: Node type of the clone.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.clone_instance(
                instance_id="example",
                name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/clone",
            body=marshal_CloneInstanceRequest(
                CloneInstanceRequest(
                    instance_id=instance_id,
                    name=name,
                    region=region,
                    node_type=node_type,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def restart_instance(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> Instance:
        """
        Restart Database Instance.
        Restart a given Database Instance, specified by the `region` and `instance_id` parameters. The status of the Database Instance returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to restart.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.restart_instance(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/restart",
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def get_instance_certificate(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> Optional[ScwFile]:
        """
        Get the TLS certificate of a Database Instance.
        Retrieve information about the TLS certificate of a given Database Instance. Details like name and content are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :return: :class:`Optional[ScwFile] <Optional[ScwFile]>`

        Usage:
        ::

            result = api.get_instance_certificate(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/certificate",
        )

        self._throw_on_error(res)
        json = res.json()
        return unmarshal_ScwFile(json) if json is not None else None

    def renew_instance_certificate(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Renew the TLS certificate of a Database Instance.
        Renew a TLS for a Database Instance. Renewing a certificate means that you will not be able to connect to your Database Instance using the previous certificate. You will also need to download and update the new certificate for all database clients.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want logs of.

        Usage:
        ::

            result = api.renew_instance_certificate(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/renew-certificate",
        )

        self._throw_on_error(res)
        return None

    def get_instance_metrics(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
        metric_name: Optional[str] = None,
    ) -> InstanceMetrics:
        """
        Get Database Instance metrics.
        Retrieve the time series metrics of a given Database Instance. You can define the period from which to retrieve metrics by specifying the `start_date` and `end_date`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param start_date: Start date to gather metrics from.
        :param end_date: End date to gather metrics from.
        :param metric_name: Name of the metric to gather.
        :return: :class:`InstanceMetrics <InstanceMetrics>`

        Usage:
        ::

            result = api.get_instance_metrics(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/metrics",
            params={
                "end_date": end_date,
                "metric_name": metric_name,
                "start_date": start_date,
            },
        )

        self._throw_on_error(res)
        return unmarshal_InstanceMetrics(res.json())

    def create_read_replica(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        endpoint_spec: Optional[List[ReadReplicaEndpointSpec]] = None,
        same_zone: Optional[bool] = None,
    ) -> ReadReplica:
        """
        Create a Read Replica.
        Create a new Read Replica of a Database Instance. You must specify the `region` and the `instance_id`. You can only create a maximum of 3 Read Replicas per Database Instance.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to create a Read Replica from.
        :param endpoint_spec: Specification of the endpoint you want to create.
        :param same_zone: Defines whether to create the replica in the same availability zone as the main instance nodes or not.
        :return: :class:`ReadReplica <ReadReplica>`

        Usage:
        ::

            result = api.create_read_replica(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/read-replicas",
            body=marshal_CreateReadReplicaRequest(
                CreateReadReplicaRequest(
                    instance_id=instance_id,
                    region=region,
                    endpoint_spec=endpoint_spec,
                    same_zone=same_zone,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_ReadReplica(res.json())

    def get_read_replica(
        self,
        *,
        read_replica_id: str,
        region: Optional[Region] = None,
    ) -> ReadReplica:
        """
        Get a Read Replica.
        Retrieve information about a Database Instance Read Replica. Full details about the Read Replica, like `endpoints`, `status`  and `region` are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param read_replica_id: UUID of the Read Replica.
        :return: :class:`ReadReplica <ReadReplica>`

        Usage:
        ::

            result = api.get_read_replica(read_replica_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_read_replica_id = validate_path_param("read_replica_id", read_replica_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/read-replicas/{param_read_replica_id}",
        )

        self._throw_on_error(res)
        return unmarshal_ReadReplica(res.json())

    def wait_for_read_replica(
        self,
        *,
        read_replica_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[ReadReplica, bool]] = None,
    ) -> ReadReplica:
        """
        Waits for :class:`ReadReplica <ReadReplica>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param read_replica_id: UUID of the Read Replica.
        :param options: The options for the waiter
        :return: :class:`ReadReplica <ReadReplica>`

        Usage:
        ::

            result = api.wait_for_read_replica(read_replica_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in READ_REPLICA_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_read_replica,
            options=options,
            args={
                "read_replica_id": read_replica_id,
                "region": region,
            },
        )

    def delete_read_replica(
        self,
        *,
        read_replica_id: str,
        region: Optional[Region] = None,
    ) -> ReadReplica:
        """
        Delete a Read Replica.
        Delete a Read Replica of a Database Instance. You must specify the `region` and `read_replica_id` parameters of the Read Replica you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param read_replica_id: UUID of the Read Replica.
        :return: :class:`ReadReplica <ReadReplica>`

        Usage:
        ::

            result = api.delete_read_replica(read_replica_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_read_replica_id = validate_path_param("read_replica_id", read_replica_id)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/read-replicas/{param_read_replica_id}",
        )

        self._throw_on_error(res)
        return unmarshal_ReadReplica(res.json())

    def reset_read_replica(
        self,
        *,
        read_replica_id: str,
        region: Optional[Region] = None,
    ) -> ReadReplica:
        """
        Resync a Read Replica.
        When you resync a Read Replica, first it is reset, then its data is resynchronized from the primary node. Your Read Replica remains unavailable during the resync process. The duration of this process is proportional to the size of your Database Instance.
        The configured endpoints do not change.
        :param region: Region to target. If none is passed will use default region from the config.
        :param read_replica_id: UUID of the Read Replica.
        :return: :class:`ReadReplica <ReadReplica>`

        Usage:
        ::

            result = api.reset_read_replica(read_replica_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_read_replica_id = validate_path_param("read_replica_id", read_replica_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/read-replicas/{param_read_replica_id}/reset",
        )

        self._throw_on_error(res)
        return unmarshal_ReadReplica(res.json())

    def create_read_replica_endpoint(
        self,
        *,
        read_replica_id: str,
        endpoint_spec: List[ReadReplicaEndpointSpec],
        region: Optional[Region] = None,
    ) -> ReadReplica:
        """
        Create an endpoint for a Read Replica.
        Create a new endpoint for a Read Replica. Read Replicas can have at most one direct access and one Private Network endpoint.
        :param region: Region to target. If none is passed will use default region from the config.
        :param read_replica_id: UUID of the Read Replica.
        :param endpoint_spec: Specification of the endpoint you want to create.
        :return: :class:`ReadReplica <ReadReplica>`

        Usage:
        ::

            result = api.create_read_replica_endpoint(
                read_replica_id="example",
                endpoint_spec=[ReadReplicaEndpointSpec(...)],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_read_replica_id = validate_path_param("read_replica_id", read_replica_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/read-replicas/{param_read_replica_id}/endpoints",
            body=marshal_CreateReadReplicaEndpointRequest(
                CreateReadReplicaEndpointRequest(
                    read_replica_id=read_replica_id,
                    endpoint_spec=endpoint_spec,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_ReadReplica(res.json())

    def prepare_instance_logs(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
    ) -> PrepareInstanceLogsResponse:
        """
        Prepare logs of a Database Instance.
        Prepare your Database Instance logs. You can define the `start_date` and `end_date` parameters for your query. The download URL is returned in the response. Logs are recorded from 00h00 to 23h59 and then aggregated in a `.log` file once a day. Therefore, even if you specify a timeframe from which you want to get the logs, you will receive logs from the full 24 hours.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want logs of.
        :param start_date: Start datetime of your log. (RFC 3339 format).
        :param end_date: End datetime of your log. (RFC 3339 format).
        :return: :class:`PrepareInstanceLogsResponse <PrepareInstanceLogsResponse>`

        Usage:
        ::

            result = api.prepare_instance_logs(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/prepare-logs",
            body=marshal_PrepareInstanceLogsRequest(
                PrepareInstanceLogsRequest(
                    instance_id=instance_id,
                    region=region,
                    start_date=start_date,
                    end_date=end_date,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_PrepareInstanceLogsResponse(res.json())

    def list_instance_logs(
        self,
        *,
        instance_id: str,
        order_by: ListInstanceLogsRequestOrderBy,
        region: Optional[Region] = None,
    ) -> ListInstanceLogsResponse:
        """
        List available logs of a Database Instance.
        List the available logs of a Database Instance. By default, the logs returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want logs of.
        :param order_by: Criteria to use when ordering Database Instance logs listing.
        :return: :class:`ListInstanceLogsResponse <ListInstanceLogsResponse>`

        Usage:
        ::

            result = api.list_instance_logs(
                instance_id="example",
                order_by=created_at_asc,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/logs",
            params={
                "order_by": order_by,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListInstanceLogsResponse(res.json())

    def get_instance_log(
        self,
        *,
        instance_log_id: str,
        region: Optional[Region] = None,
    ) -> InstanceLog:
        """
        Get given logs of a Database Instance.
        Retrieve information about the logs of a Database Instance. Specify the `instance_log_id` and `region` in your request to get information such as `download_url`, `status`, `expires_at` and `created_at` about your logs in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_log_id: UUID of the instance_log you want.
        :return: :class:`InstanceLog <InstanceLog>`

        Usage:
        ::

            result = api.get_instance_log(instance_log_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_log_id = validate_path_param("instance_log_id", instance_log_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/logs/{param_instance_log_id}",
        )

        self._throw_on_error(res)
        return unmarshal_InstanceLog(res.json())

    def wait_for_instance_log(
        self,
        *,
        instance_log_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[InstanceLog, bool]] = None,
    ) -> InstanceLog:
        """
        Waits for :class:`InstanceLog <InstanceLog>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_log_id: UUID of the instance_log you want.
        :param options: The options for the waiter
        :return: :class:`InstanceLog <InstanceLog>`

        Usage:
        ::

            result = api.wait_for_instance_log(instance_log_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in INSTANCE_LOG_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_instance_log,
            options=options,
            args={
                "instance_log_id": instance_log_id,
                "region": region,
            },
        )

    def purge_instance_logs(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        log_name: Optional[str] = None,
    ) -> Optional[None]:
        """
        Purge remote Database Instance logs.
        Purge a given remote log from a Database Instance. You can specify the `log_name` of the log you wish to clean from your Database Instance.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want logs of.
        :param log_name: Given log name to purge.

        Usage:
        ::

            result = api.purge_instance_logs(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/purge-logs",
            body=marshal_PurgeInstanceLogsRequest(
                PurgeInstanceLogsRequest(
                    instance_id=instance_id,
                    region=region,
                    log_name=log_name,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return None

    def list_instance_logs_details(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> ListInstanceLogsDetailsResponse:
        """
        List remote Database Instance logs details.
        List remote log details. By default, the details returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want logs of.
        :return: :class:`ListInstanceLogsDetailsResponse <ListInstanceLogsDetailsResponse>`

        Usage:
        ::

            result = api.list_instance_logs_details(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/logs-details",
        )

        self._throw_on_error(res)
        return unmarshal_ListInstanceLogsDetailsResponse(res.json())

    def add_instance_settings(
        self,
        *,
        instance_id: str,
        settings: List[InstanceSetting],
        region: Optional[Region] = None,
    ) -> AddInstanceSettingsResponse:
        """
        Add Database Instance advanced settings.
        Add an advanced setting to a Database Instance. You must set the `name` and the `value` of each setting.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to add settings to.
        :param settings: Settings to add to the Database Instance.
        :return: :class:`AddInstanceSettingsResponse <AddInstanceSettingsResponse>`

        Usage:
        ::

            result = api.add_instance_settings(
                instance_id="example",
                settings=[InstanceSetting(...)],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/settings",
            body=marshal_AddInstanceSettingsRequest(
                AddInstanceSettingsRequest(
                    instance_id=instance_id,
                    settings=settings,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_AddInstanceSettingsResponse(res.json())

    def delete_instance_settings(
        self,
        *,
        instance_id: str,
        setting_names: List[str],
        region: Optional[Region] = None,
    ) -> DeleteInstanceSettingsResponse:
        """
        Delete Database Instance advanced settings.
        Delete an advanced setting in a Database Instance. You must specify the names of the settings you want to delete in the request.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance to delete settings from.
        :param setting_names: Settings names to delete.
        :return: :class:`DeleteInstanceSettingsResponse <DeleteInstanceSettingsResponse>`

        Usage:
        ::

            result = api.delete_instance_settings(
                instance_id="example",
                setting_names=["example"],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/settings",
            body=marshal_DeleteInstanceSettingsRequest(
                DeleteInstanceSettingsRequest(
                    instance_id=instance_id,
                    setting_names=setting_names,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DeleteInstanceSettingsResponse(res.json())

    def set_instance_settings(
        self,
        *,
        instance_id: str,
        settings: List[InstanceSetting],
        region: Optional[Region] = None,
    ) -> SetInstanceSettingsResponse:
        """
        Set Database Instance advanced settings.
        Update an advanced setting for a Database Instance. Settings added upon database engine initalization can only be defined once, and cannot, therefore, be updated.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance where the settings must be set.
        :param settings: Settings to define for the Database Instance.
        :return: :class:`SetInstanceSettingsResponse <SetInstanceSettingsResponse>`

        Usage:
        ::

            result = api.set_instance_settings(
                instance_id="example",
                settings=[InstanceSetting(...)],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "PUT",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/settings",
            body=marshal_SetInstanceSettingsRequest(
                SetInstanceSettingsRequest(
                    instance_id=instance_id,
                    settings=settings,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_SetInstanceSettingsResponse(res.json())

    def list_instance_acl_rules(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListInstanceACLRulesResponse:
        """
        List ACL rules of a Database Instance.
        List the ACL rules for a given Database Instance. The response is an array of ACL objects, each one representing an ACL that denies, allows or redirects traffic based on certain conditions.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param page:
        :param page_size:
        :return: :class:`ListInstanceACLRulesResponse <ListInstanceACLRulesResponse>`

        Usage:
        ::

            result = api.list_instance_acl_rules(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/acls",
            params={
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListInstanceACLRulesResponse(res.json())

    def list_instance_acl_rules_all(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[ACLRule]:
        """
        List ACL rules of a Database Instance.
        List the ACL rules for a given Database Instance. The response is an array of ACL objects, each one representing an ACL that denies, allows or redirects traffic based on certain conditions.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param page:
        :param page_size:
        :return: :class:`List[ListInstanceACLRulesResponse] <List[ListInstanceACLRulesResponse]>`

        Usage:
        ::

            result = api.list_instance_acl_rules_all(instance_id="example")
        """

        return fetch_all_pages(
            type=ListInstanceACLRulesResponse,
            key="rules",
            fetcher=self.list_instance_acl_rules,
            args={
                "instance_id": instance_id,
                "region": region,
                "page": page,
                "page_size": page_size,
            },
        )

    def add_instance_acl_rules(
        self,
        *,
        instance_id: str,
        rules: List[ACLRuleRequest],
        region: Optional[Region] = None,
    ) -> AddInstanceACLRulesResponse:
        """
        Add an ACL rule to a Database Instance.
        Add an additional ACL rule to a Database Instance.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to add ACL rules to.
        :param rules: ACL rules to add to the Database Instance.
        :return: :class:`AddInstanceACLRulesResponse <AddInstanceACLRulesResponse>`

        Usage:
        ::

            result = api.add_instance_acl_rules(
                instance_id="example",
                rules=[ACLRuleRequest(...)],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/acls",
            body=marshal_AddInstanceACLRulesRequest(
                AddInstanceACLRulesRequest(
                    instance_id=instance_id,
                    rules=rules,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_AddInstanceACLRulesResponse(res.json())

    def set_instance_acl_rules(
        self,
        *,
        instance_id: str,
        rules: List[ACLRuleRequest],
        region: Optional[Region] = None,
    ) -> SetInstanceACLRulesResponse:
        """
        Set ACL rules for a Database Instance.
        Replace all the ACL rules of a Database Instance.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance where the ACL rules must be set.
        :param rules: ACL rules to define for the Database Instance.
        :return: :class:`SetInstanceACLRulesResponse <SetInstanceACLRulesResponse>`

        Usage:
        ::

            result = api.set_instance_acl_rules(
                instance_id="example",
                rules=[ACLRuleRequest(...)],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "PUT",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/acls",
            body=marshal_SetInstanceACLRulesRequest(
                SetInstanceACLRulesRequest(
                    instance_id=instance_id,
                    rules=rules,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_SetInstanceACLRulesResponse(res.json())

    def delete_instance_acl_rules(
        self,
        *,
        instance_id: str,
        acl_rule_ips: List[str],
        region: Optional[Region] = None,
    ) -> DeleteInstanceACLRulesResponse:
        """
        Delete ACL rules of a Database Instance.
        Delete one or more ACL rules of a Database Instance.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you want to delete an ACL rule from.
        :param acl_rule_ips: IP addresses defined in the ACL rules of the Database Instance.
        :return: :class:`DeleteInstanceACLRulesResponse <DeleteInstanceACLRulesResponse>`

        Usage:
        ::

            result = api.delete_instance_acl_rules(
                instance_id="example",
                acl_rule_ips=["example"],
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/acls",
            body=marshal_DeleteInstanceACLRulesRequest(
                DeleteInstanceACLRulesRequest(
                    instance_id=instance_id,
                    acl_rule_ips=acl_rule_ips,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_DeleteInstanceACLRulesResponse(res.json())

    def list_users(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        order_by: ListUsersRequestOrderBy = ListUsersRequestOrderBy.NAME_ASC,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListUsersResponse:
        """
        List users of a Database Instance.
        List all users of a given Database Instance. By default, the users returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param name: Name of the user.
        :param order_by: Criteria to use when requesting user listing.
        :param page:
        :param page_size:
        :return: :class:`ListUsersResponse <ListUsersResponse>`

        Usage:
        ::

            result = api.list_users(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/users",
            params={
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListUsersResponse(res.json())

    def list_users_all(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        order_by: Optional[ListUsersRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[User]:
        """
        List users of a Database Instance.
        List all users of a given Database Instance. By default, the users returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param name: Name of the user.
        :param order_by: Criteria to use when requesting user listing.
        :param page:
        :param page_size:
        :return: :class:`List[ListUsersResponse] <List[ListUsersResponse]>`

        Usage:
        ::

            result = api.list_users_all(instance_id="example")
        """

        return fetch_all_pages(
            type=ListUsersResponse,
            key="users",
            fetcher=self.list_users,
            args={
                "instance_id": instance_id,
                "region": region,
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
            },
        )

    def create_user(
        self,
        *,
        instance_id: str,
        name: str,
        password: str,
        is_admin: bool,
        region: Optional[Region] = None,
    ) -> User:
        """
        Create a user for a Database Instance.
        Create a new user for a Database Instance. You must define the `name`, `password` and `is_admin` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance in which you want to create a user.
        :param name: Name of the user you want to create.
        :param password: Password of the user you want to create.
        :param is_admin: Defines whether the user will have administrative privileges.
        :return: :class:`User <User>`

        Usage:
        ::

            result = api.create_user(
                instance_id="example",
                name="example",
                password="example",
                is_admin=True,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/users",
            body=marshal_CreateUserRequest(
                CreateUserRequest(
                    instance_id=instance_id,
                    name=name,
                    password=password,
                    is_admin=is_admin,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_User(res.json())

    def update_user(
        self,
        *,
        instance_id: str,
        name: str,
        region: Optional[Region] = None,
        password: Optional[str] = None,
        is_admin: Optional[bool] = None,
    ) -> User:
        """
        Update a user on a Database Instance.
        Update the parameters of a user on a Database Instance. You can update the `password` and `is_admin` parameters, but you cannot change the name of the user.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance the user belongs to.
        :param name: Name of the database user.
        :param password: Password of the database user.
        :param is_admin: Defines whether or not this user got administrative privileges.
        :return: :class:`User <User>`

        Usage:
        ::

            result = api.update_user(
                instance_id="example",
                name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)
        param_name = validate_path_param("name", name)

        res = self._request(
            "PATCH",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/users/{param_name}",
            body=marshal_UpdateUserRequest(
                UpdateUserRequest(
                    instance_id=instance_id,
                    name=name,
                    region=region,
                    password=password,
                    is_admin=is_admin,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_User(res.json())

    def delete_user(
        self,
        *,
        instance_id: str,
        name: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Delete a user on a Database Instance.
        Delete a given user on a Database Instance. You must specify, in the endpoint,  the `region`, `instance_id` and `name` parameters of the user you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance to delete the user from.
        :param name: Name of the user.

        Usage:
        ::

            result = api.delete_user(
                instance_id="example",
                name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)
        param_name = validate_path_param("name", name)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/users/{param_name}",
        )

        self._throw_on_error(res)
        return None

    def list_databases(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        managed: Optional[bool] = None,
        owner: Optional[str] = None,
        order_by: ListDatabasesRequestOrderBy = ListDatabasesRequestOrderBy.NAME_ASC,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListDatabasesResponse:
        """
        List databases in a Database Instance.
        List all databases of a given Database Instance. By default, the databases returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field. You can define additional parameters for your query, such as `name`, `managed` and `owner`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance to list the databases of.
        :param name: Name of the database.
        :param managed: Defines whether or not the database is managed.
        :param owner: User that owns this database.
        :param order_by: Criteria to use when ordering database listing.
        :param page:
        :param page_size:
        :return: :class:`ListDatabasesResponse <ListDatabasesResponse>`

        Usage:
        ::

            result = api.list_databases(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/databases",
            params={
                "managed": managed,
                "name": name,
                "order_by": order_by,
                "owner": owner,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListDatabasesResponse(res.json())

    def list_databases_all(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        managed: Optional[bool] = None,
        owner: Optional[str] = None,
        order_by: Optional[ListDatabasesRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[Database]:
        """
        List databases in a Database Instance.
        List all databases of a given Database Instance. By default, the databases returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field. You can define additional parameters for your query, such as `name`, `managed` and `owner`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance to list the databases of.
        :param name: Name of the database.
        :param managed: Defines whether or not the database is managed.
        :param owner: User that owns this database.
        :param order_by: Criteria to use when ordering database listing.
        :param page:
        :param page_size:
        :return: :class:`List[ListDatabasesResponse] <List[ListDatabasesResponse]>`

        Usage:
        ::

            result = api.list_databases_all(instance_id="example")
        """

        return fetch_all_pages(
            type=ListDatabasesResponse,
            key="databases",
            fetcher=self.list_databases,
            args={
                "instance_id": instance_id,
                "region": region,
                "name": name,
                "managed": managed,
                "owner": owner,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
            },
        )

    def create_database(
        self,
        *,
        instance_id: str,
        name: str,
        region: Optional[Region] = None,
    ) -> Database:
        """
        Create a database in a Database Instance.
        Create a new database. You must define the `name` parameter in the request.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance where to create the database.
        :param name: Name of the database.
        :return: :class:`Database <Database>`

        Usage:
        ::

            result = api.create_database(
                instance_id="example",
                name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/databases",
            body=marshal_CreateDatabaseRequest(
                CreateDatabaseRequest(
                    instance_id=instance_id,
                    name=name,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Database(res.json())

    def delete_database(
        self,
        *,
        instance_id: str,
        name: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Delete a database in a Database Instance.
        Delete a given database on a Database Instance. You must specify, in the endpoint, the `region`, `instance_id` and `name` parameters of the database you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance where to delete the database.
        :param name: Name of the database to delete.

        Usage:
        ::

            result = api.delete_database(
                instance_id="example",
                name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)
        param_name = validate_path_param("name", name)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/databases/{param_name}",
        )

        self._throw_on_error(res)
        return None

    def list_privileges(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        order_by: ListPrivilegesRequestOrderBy = ListPrivilegesRequestOrderBy.USER_NAME_ASC,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        database_name: Optional[str] = None,
        user_name: Optional[str] = None,
    ) -> ListPrivilegesResponse:
        """
        List user privileges for a database.
        List privileges of a user on a database. By default, the details returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field. You can define additional parameters for your query, such as `database_name` and `user_name`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param order_by: Criteria to use when ordering privileges listing.
        :param page:
        :param page_size:
        :param database_name: Name of the database.
        :param user_name: Name of the user.
        :return: :class:`ListPrivilegesResponse <ListPrivilegesResponse>`

        Usage:
        ::

            result = api.list_privileges(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/privileges",
            params={
                "database_name": database_name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "user_name": user_name,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListPrivilegesResponse(res.json())

    def list_privileges_all(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        order_by: Optional[ListPrivilegesRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        database_name: Optional[str] = None,
        user_name: Optional[str] = None,
    ) -> List[Privilege]:
        """
        List user privileges for a database.
        List privileges of a user on a database. By default, the details returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field. You can define additional parameters for your query, such as `database_name` and `user_name`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param order_by: Criteria to use when ordering privileges listing.
        :param page:
        :param page_size:
        :param database_name: Name of the database.
        :param user_name: Name of the user.
        :return: :class:`List[ListPrivilegesResponse] <List[ListPrivilegesResponse]>`

        Usage:
        ::

            result = api.list_privileges_all(instance_id="example")
        """

        return fetch_all_pages(
            type=ListPrivilegesResponse,
            key="privileges",
            fetcher=self.list_privileges,
            args={
                "instance_id": instance_id,
                "region": region,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
                "database_name": database_name,
                "user_name": user_name,
            },
        )

    def set_privilege(
        self,
        *,
        instance_id: str,
        database_name: str,
        user_name: str,
        permission: Permission,
        region: Optional[Region] = None,
    ) -> Privilege:
        """
        Set user privileges for a database.
        Set the privileges of a user on a database. You must define `database_name`, `user_name` and `permission` in the request body.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param database_name: Name of the database.
        :param user_name: Name of the user.
        :param permission: Permission to set (Read, Read/Write, All, Custom).
        :return: :class:`Privilege <Privilege>`

        Usage:
        ::

            result = api.set_privilege(
                instance_id="example",
                database_name="example",
                user_name="example",
                permission=readonly,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "PUT",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/privileges",
            body=marshal_SetPrivilegeRequest(
                SetPrivilegeRequest(
                    instance_id=instance_id,
                    database_name=database_name,
                    user_name=user_name,
                    permission=permission,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Privilege(res.json())

    def list_snapshots(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        order_by: ListSnapshotsRequestOrderBy = ListSnapshotsRequestOrderBy.CREATED_AT_ASC,
        instance_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListSnapshotsResponse:
        """
        List snapshots.
        List snapshots. You can include the `instance_id` or `project_id` in your query to get the list of snapshots for specific Database Instances and/or Projects. By default, the details returned in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the snapshot.
        :param order_by: Criteria to use when ordering snapshot listing.
        :param instance_id: UUID of the Database Instance.
        :param organization_id: Organization ID the snapshots belongs to.
        :param project_id: Project ID the snapshots belongs to.
        :param page:
        :param page_size:
        :return: :class:`ListSnapshotsResponse <ListSnapshotsResponse>`

        Usage:
        ::

            result = api.list_snapshots()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/snapshots",
            params={
                "instance_id": instance_id,
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListSnapshotsResponse(res.json())

    def list_snapshots_all(
        self,
        *,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        order_by: Optional[ListSnapshotsRequestOrderBy] = None,
        instance_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[Snapshot]:
        """
        List snapshots.
        List snapshots. You can include the `instance_id` or `project_id` in your query to get the list of snapshots for specific Database Instances and/or Projects. By default, the details returned in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the snapshot.
        :param order_by: Criteria to use when ordering snapshot listing.
        :param instance_id: UUID of the Database Instance.
        :param organization_id: Organization ID the snapshots belongs to.
        :param project_id: Project ID the snapshots belongs to.
        :param page:
        :param page_size:
        :return: :class:`List[ListSnapshotsResponse] <List[ListSnapshotsResponse]>`

        Usage:
        ::

            result = api.list_snapshots_all()
        """

        return fetch_all_pages(
            type=ListSnapshotsResponse,
            key="snapshots",
            fetcher=self.list_snapshots,
            args={
                "region": region,
                "name": name,
                "order_by": order_by,
                "instance_id": instance_id,
                "organization_id": organization_id,
                "project_id": project_id,
                "page": page,
                "page_size": page_size,
            },
        )

    def get_snapshot(
        self,
        *,
        snapshot_id: str,
        region: Optional[Region] = None,
    ) -> Snapshot:
        """
        Get a Database Instance snapshot.
        Retrieve information about a given snapshot, specified by its `snapshot_id` and `region`. Full details about the snapshot, like size and expiration date, are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param snapshot_id: UUID of the snapshot.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.get_snapshot(snapshot_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/snapshots/{param_snapshot_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def wait_for_snapshot(
        self,
        *,
        snapshot_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Snapshot, bool]] = None,
    ) -> Snapshot:
        """
        Waits for :class:`Snapshot <Snapshot>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param snapshot_id: UUID of the snapshot.
        :param options: The options for the waiter
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.wait_for_snapshot(snapshot_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in SNAPSHOT_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_snapshot,
            options=options,
            args={
                "snapshot_id": snapshot_id,
                "region": region,
            },
        )

    def create_snapshot(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        expires_at: Optional[datetime] = None,
    ) -> Snapshot:
        """
        Create a Database Instance snapshot.
        Create a new snapshot of a Database Instance. You must define the `name` parameter in the request.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance.
        :param name: Name of the snapshot.
        :param expires_at: Expiration date (must follow the ISO 8601 format).
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.create_snapshot(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/snapshots",
            body=marshal_CreateSnapshotRequest(
                CreateSnapshotRequest(
                    instance_id=instance_id,
                    region=region,
                    name=name or random_name(prefix="snp"),
                    expires_at=expires_at,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def update_snapshot(
        self,
        *,
        snapshot_id: str,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        expires_at: Optional[datetime] = None,
    ) -> Snapshot:
        """
        Update a Database Instance snapshot.
        Update the parameters of a snapshot of a Database Instance. You can update the `name` and `expires_at` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param snapshot_id: UUID of the snapshot to update.
        :param name: Name of the snapshot.
        :param expires_at: Expiration date (must follow the ISO 8601 format).
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.update_snapshot(snapshot_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "PATCH",
            f"/rdb/v1/regions/{param_region}/snapshots/{param_snapshot_id}",
            body=marshal_UpdateSnapshotRequest(
                UpdateSnapshotRequest(
                    snapshot_id=snapshot_id,
                    region=region,
                    name=name,
                    expires_at=expires_at,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def delete_snapshot(
        self,
        *,
        snapshot_id: str,
        region: Optional[Region] = None,
    ) -> Snapshot:
        """
        Delete a Database Instance snapshot.
        Delete a given snapshot of a Database Instance. You must specify, in the endpoint,  the `region` and `snapshot_id` parameters of the snapshot you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param snapshot_id: UUID of the snapshot to delete.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.delete_snapshot(snapshot_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/snapshots/{param_snapshot_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def create_instance_from_snapshot(
        self,
        *,
        snapshot_id: str,
        instance_name: str,
        region: Optional[Region] = None,
        is_ha_cluster: Optional[bool] = None,
        node_type: Optional[str] = None,
    ) -> Instance:
        """
        Create a new Database Instance from a snapshot.
        Restore a snapshot. When you restore a snapshot, a new Instance is created and billed to your account. Note that is possible to select a larger node type for your new Database Instance. However, the Block volume size will be the same as the size of the restored snapshot. All Instance settings will be restored if you chose a node type with the same or more memory size than the initial Instance. Settings will be reset to the default if your node type has less memory.
        :param region: Region to target. If none is passed will use default region from the config.
        :param snapshot_id: Block snapshot of the Database Instance.
        :param instance_name: Name of the Database Instance created with the snapshot.
        :param is_ha_cluster: Defines whether or not High-Availability is enabled on the new Database Instance.
        :param node_type: The node type used to restore the snapshot.
        :return: :class:`Instance <Instance>`

        Usage:
        ::

            result = api.create_instance_from_snapshot(
                snapshot_id="example",
                instance_name="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/snapshots/{param_snapshot_id}/create-instance",
            body=marshal_CreateInstanceFromSnapshotRequest(
                CreateInstanceFromSnapshotRequest(
                    snapshot_id=snapshot_id,
                    instance_name=instance_name,
                    region=region,
                    is_ha_cluster=is_ha_cluster,
                    node_type=node_type,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Instance(res.json())

    def create_endpoint(
        self,
        *,
        instance_id: str,
        region: Optional[Region] = None,
        endpoint_spec: Optional[EndpointSpec] = None,
    ) -> Endpoint:
        """
        Create a new Database Instance endpoint.
        Create a new endpoint for a Database Instance. You can add `load_balancer` and `private_network` specifications to the body of the request. Note that this action replaces your current endpoint, which means you might need to update any environment configurations that point to the old endpoint.
        :param region: Region to target. If none is passed will use default region from the config.
        :param instance_id: UUID of the Database Instance you to which you want to add an endpoint.
        :param endpoint_spec: Specification of the endpoint you want to create.
        :return: :class:`Endpoint <Endpoint>`

        Usage:
        ::

            result = api.create_endpoint(instance_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_instance_id = validate_path_param("instance_id", instance_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/instances/{param_instance_id}/endpoints",
            body=marshal_CreateEndpointRequest(
                CreateEndpointRequest(
                    instance_id=instance_id,
                    region=region,
                    endpoint_spec=endpoint_spec,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Endpoint(res.json())

    def delete_endpoint(
        self,
        *,
        endpoint_id: str,
        region: Optional[Region] = None,
    ) -> Optional[None]:
        """
        Delete a Database Instance endpoint.
        Delete the endpoint of a Database Instance. You must specify the `region` and `endpoint_id` parameters of the endpoint you want to delete. Note that might need to update any environment configurations that point to the deleted endpoint.
        :param region: Region to target. If none is passed will use default region from the config.
        :param endpoint_id: UUID of the endpoint you want to delete.
        This endpoint can also be used to delete a Read Replica endpoint.

        Usage:
        ::

            result = api.delete_endpoint(endpoint_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_endpoint_id = validate_path_param("endpoint_id", endpoint_id)

        res = self._request(
            "DELETE",
            f"/rdb/v1/regions/{param_region}/endpoints/{param_endpoint_id}",
        )

        self._throw_on_error(res)
        return None

    def get_endpoint(
        self,
        *,
        endpoint_id: str,
        region: Optional[Region] = None,
    ) -> Endpoint:
        """
        Get a Database Instance endpoint.
        Retrieve information about a Database Instance endpoint. Full details about the endpoint, like `ip`, `port`, `private_network` and `load_balancer` specifications are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param endpoint_id: UUID of the endpoint you want to get.
        :return: :class:`Endpoint <Endpoint>`

        Usage:
        ::

            result = api.get_endpoint(endpoint_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_endpoint_id = validate_path_param("endpoint_id", endpoint_id)

        res = self._request(
            "GET",
            f"/rdb/v1/regions/{param_region}/endpoints/{param_endpoint_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Endpoint(res.json())

    def migrate_endpoint(
        self,
        *,
        endpoint_id: str,
        instance_id: str,
        region: Optional[Region] = None,
    ) -> Endpoint:
        """
        Migrate an existing instance endpoint to another instance.
        :param region: Region to target. If none is passed will use default region from the config.
        :param endpoint_id: UUID of the endpoint you want to migrate.
        :param instance_id: UUID of the instance you want to attach the endpoint to.
        :return: :class:`Endpoint <Endpoint>`

        Usage:
        ::

            result = api.migrate_endpoint(
                endpoint_id="example",
                instance_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_endpoint_id = validate_path_param("endpoint_id", endpoint_id)

        res = self._request(
            "POST",
            f"/rdb/v1/regions/{param_region}/endpoints/{param_endpoint_id}/migrate",
            body=marshal_MigrateEndpointRequest(
                MigrateEndpointRequest(
                    endpoint_id=endpoint_id,
                    instance_id=instance_id,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Endpoint(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    HubStatus,
)


HUB_TRANSIENT_STATUSES: List[HubStatus] = [
    HubStatus.ENABLING,
    HubStatus.DISABLING,
]
"""
Lists transient statutes of the enum :class:`HubStatus <HubStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Zone,
)
from scaleway_core.utils import (
    WaitForOptions,
    fetch_all_pages,
    random_name,
    validate_path_param,
    wait_for_resource,
)
from .types import (
    ListServersRequestOrderBy,
    ListOSResponse,
    ListServerTypesResponse,
    ListServersResponse,
    OS,
    Server,
    ServerType,
    CreateServerRequest,
    UpdateServerRequest,
)
from .content import (
    SERVER_TRANSIENT_STATUSES,
)
from .marshalling import (
    marshal_CreateServerRequest,
    marshal_UpdateServerRequest,
    unmarshal_OS,
    unmarshal_Server,
    unmarshal_ServerType,
    unmarshal_ListOSResponse,
    unmarshal_ListServerTypesResponse,
    unmarshal_ListServersResponse,
)


class ApplesiliconV1Alpha1API(API):
    """
    Apple silicon API.

    Apple silicon API.
    """

    def list_server_types(
        self,
        *,
        zone: Optional[Zone] = None,
    ) -> ListServerTypesResponse:
        """
        List server types.
        List all technical details about Apple silicon server types available in the specified zone. Since there is only one Availability Zone for Apple silicon servers, the targeted value is `fr-par-3`.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :return: :class:`ListServerTypesResponse <ListServerTypesResponse>`

        Usage:
        ::

            result = api.list_server_types()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "GET",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/server-types",
        )

        self._throw_on_error(res)
        return unmarshal_ListServerTypesResponse(res.json())

    def get_server_type(
        self,
        *,
        server_type: str,
        zone: Optional[Zone] = None,
    ) -> ServerType:
        """
        Get a server type.
        Get technical details (CPU, disk size etc.) of a server type.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_type: Server type identifier.
        :return: :class:`ServerType <ServerType>`

        Usage:
        ::

            result = api.get_server_type(server_type="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_server_type = validate_path_param("server_type", server_type)

        res = self._request(
            "GET",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/server-type/{param_server_type}",
        )

        self._throw_on_error(res)
        return unmarshal_ServerType(res.json())

    def create_server(
        self,
        *,
        type_: str,
        zone: Optional[Zone] = None,
        name: Optional[str] = None,
        project_id: Optional[str] = None,
    ) -> Server:
        """
        Create a server.
        Create a new server in the targeted zone, specifying its configuration including name and type.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param name: Create a server with this given name.
        :param project_id: Create a server in the given project ID.
        :param type_: Create a server of the given type.
        :return: :class:`Server <Server>`

        Usage:
        ::

            result = api.create_server(type_="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "POST",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers",
            body=marshal_CreateServerRequest(
                CreateServerRequest(
                    type_=type_,
                    zone=zone,
                    name=name or random_name(prefix="as"),
                    project_id=project_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Server(res.json())

    def list_servers(
        self,
        *,
        zone: Optional[Zone] = None,
        order_by: ListServersRequestOrderBy = ListServersRequestOrderBy.CREATED_AT_ASC,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListServersResponse:
        """
        List all servers.
        List all servers in the specified zone. By default, returned servers in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param order_by: Sort order of the returned servers.
        :param project_id: Only list servers of this project ID.
        :param organization_id: Only list servers of this Organization ID.
        :param page: Positive integer to choose the page to return.
        :param page_size: Positive integer lower or equal to 100 to select the number of items to return.
        :return: :class:`ListServersResponse <ListServersResponse>`

        Usage:
        ::

            result = api.list_servers()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "GET",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers",
            params={
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListServersResponse(res.json())

    def list_servers_all(
        self,
        *,
        zone: Optional[Zone] = None,
        order_by: Optional[ListServersRequestOrderBy] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[Server]:
        """
        List all servers.
        List all servers in the specified zone. By default, returned servers in the list are ordered by creation date in ascending order, though this can be modified via the `order_by` field.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param order_by: Sort order of the returned servers.
        :param project_id: Only list servers of this project ID.
        :param organization_id: Only list servers of this Organization ID.
        :param page: Positive integer to choose the page to return.
        :param page_size: Positive integer lower or equal to 100 to select the number of items to return.
        :return: :class:`List[ListServersResponse] <List[ListServersResponse]>`

        Usage:
        ::

            result = api.list_servers_all()
        """

        return fetch_all_pages(
            type=ListServersResponse,
            key="servers",
            fetcher=self.list_servers,
            args={
                "zone": zone,
                "order_by": order_by,
                "project_id": project_id,
                "organization_id": organization_id,
                "page": page,
                "page_size": page_size,
            },
        )

    def list_os(
        self,
        *,
        zone: Optional[Zone] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        server_type: Optional[str] = None,
        name: Optional[str] = None,
    ) -> ListOSResponse:
        """
        List all Operating Systems (OS).
        List all Operating Systems (OS). The response will include the total number of OS as well as their associated IDs, names and labels.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param page: Positive integer to choose the page to return.
        :param page_size: Positive integer lower or equal to 100 to select the number of items to return.
        :param server_type: List of compatible server types.
        :param name: Filter OS by name (note that "11.1" will return "11.1.2" and "11.1" but not "12")).
        :return: :class:`ListOSResponse <ListOSResponse>`

        Usage:
        ::

            result = api.list_os()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "GET",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/os",
            params={
                "name": name,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "server_type": server_type,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListOSResponse(res.json())

    def list_os_all(
        self,
        *,
        zone: Optional[Zone] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        server_type: Optional[str] = None,
        name: Optional[str] = None,
    ) -> List[OS]:
        """
        List all Operating Systems (OS).
        List all Operating Systems (OS). The response will include the total number of OS as well as their associated IDs, names and labels.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param page: Positive integer to choose the page to return.
        :param page_size: Positive integer lower or equal to 100 to select the number of items to return.
        :param server_type: List of compatible server types.
        :param name: Filter OS by name (note that "11.1" will return "11.1.2" and "11.1" but not "12")).
        :return: :class:`List[ListOSResponse] <List[ListOSResponse]>`

        Usage:
        ::

            result = api.list_os_all()
        """

        return fetch_all_pages(
            type=ListOSResponse,
            key="os",
            fetcher=self.list_os,
            args={
                "zone": zone,
                "page": page,
                "page_size": page_size,
                "server_type": server_type,
                "name": name,
            },
        )

    def get_os(
        self,
        *,
        os_id: str,
        zone: Optional[Zone] = None,
    ) -> OS:
        """
        Get an Operating System (OS).
        Get an Operating System (OS).  The response will include the OS's unique ID as well as its name and label.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param os_id: UUID of the OS you want to get.
        :return: :class:`OS <OS>`

        Usage:
        ::

            result = api.get_os(os_id="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_os_id = validate_path_param("os_id", os_id)

        res = self._request(
            "GET",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/os/{param_os_id}",
        )

        self._throw_on_error(res)
        return unmarshal_OS(res.json())

    def get_server(
        self,
        *,
        server_id: str,
        zone: Optional[Zone] = None,
    ) -> Server:
        """
        Get a server.
        Retrieve information about an existing Apple silicon server, specified by its server ID. Its full details, including name, status and IP address, are returned in the response object.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_id: UUID of the server you want to get.
        :return: :class:`Server <Server>`

        Usage:
        ::

            result = api.get_server(server_id="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_server_id = validate_path_param("server_id", server_id)

        res = self._request(
            "GET",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers/{param_server_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Server(res.json())

    def wait_for_server(
        self,
        *,
        server_id: str,
        zone: Optional[Zone] = None,
        options: Optional[WaitForOptions[Server, bool]] = None,
    ) -> Server:
        """
        Waits for :class:`Server <Server>` to be in a final state.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_id: UUID of the server you want to get.
        :param options: The options for the waiter
        :return: :class:`Server <Server>`

        Usage:
        ::

            result = api.wait_for_server(server_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in SERVER_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_server,
            options=options,
            args={
                "server_id": server_id,
                "zone": zone,
            },
        )

    def update_server(
        self,
        *,
        server_id: str,
        name: str,
        zone: Optional[Zone] = None,
    ) -> Server:
        """
        Update a server.
        Update the parameters of an existing Apple silicon server, specified by its server ID.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_id: UUID of the server you want to update.
        :param name: Updated name for your server.
        :return: :class:`Server <Server>`

        Usage:
        ::

            result = api.update_server(
                server_id="example",
                name="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_server_id = validate_path_param("server_id", server_id)

        res = self._request(
            "PATCH",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers/{param_server_id}",
            body=marshal_UpdateServerRequest(
                UpdateServerRequest(
                    server_id=server_id,
                    name=name,
                    zone=zone,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Server(res.json())

    def delete_server(
        self,
        *,
        server_id: str,
        zone: Optional[Zone] = None,
    ) -> Optional[None]:
        """
        Delete a server.
        Delete an existing Apple silicon server, specified by its server ID. Deleting a server is permanent, and cannot be undone. Note that the minimum allocation period for Apple silicon-as-a-service is 24 hours, meaning you cannot delete your server prior to that.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_id: UUID of the server you want to delete.

        Usage:
        ::

            result = api.delete_server(server_id="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_server_id = validate_path_param("server_id", server_id)

        res = self._request(
            "DELETE",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers/{param_server_id}",
        )

        self._throw_on_error(res)
        return None

    def reboot_server(
        self,
        *,
        server_id: str,
        zone: Optional[Zone] = None,
    ) -> Server:
        """
        Reboot a server.
        Reboot an existing Apple silicon server, specified by its server ID.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_id: UUID of the server you want to reboot.
        :return: :class:`Server <Server>`

        Usage:
        ::

            result = api.reboot_server(server_id="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_server_id = validate_path_param("server_id", server_id)

        res = self._request(
            "POST",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers/{param_server_id}/reboot",
        )

        self._throw_on_error(res)
        return unmarshal_Server(res.json())

    def reinstall_server(
        self,
        *,
        server_id: str,
        zone: Optional[Zone] = None,
    ) -> Server:
        """
        Reinstall a server.
        Reinstall an existing Apple silicon server (specified by its server ID) from a new image (OS). All the data on the disk is deleted and all configuration is reset to the defailt configuration values of the image (OS).
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param server_id: UUID of the server you want to reinstall.
        :return: :class:`Server <Server>`

        Usage:
        ::

            result = api.reinstall_server(server_id="example")
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_server_id = validate_path_param("server_id", server_id)

        res = self._request(
            "POST",
            f"/apple-silicon/v1alpha1/zones/{param_zone}/servers/{param_server_id}/reinstall",
        )

        self._throw_on_error(res)
        return unmarshal_Server(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Region,
)


class ListPrivateNetworksRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListVPCsRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class AddSubnetsResponse:
    subnets: List[str]


@dataclass
class DeleteSubnetsResponse:
    subnets: List[str]


@dataclass
class ListPrivateNetworksResponse:
    private_networks: List[PrivateNetwork]

    total_count: int


@dataclass
class ListVPCsResponse:
    vpcs: List[VPC]

    total_count: int


@dataclass
class PrivateNetwork:
    """
    Private network.
    """

    id: str
    """
    Private Network ID.
    """

    name: str
    """
    Private Network name.
    """

    organization_id: str
    """
    Scaleway Organization the Private Network belongs to.
    """

    project_id: str
    """
    Scaleway Project the Private Network belongs to.
    """

    region: Region
    """
    Region in which the Private Network is available.
    """

    tags: List[str]
    """
    Tags of the Private Network.
    """

    created_at: Optional[datetime]
    """
    Date the Private Network was created.
    """

    updated_at: Optional[datetime]
    """
    Date the Private Network was last modified.
    """

    subnets: List[Subnet]
    """
    Private Network subnets.
    """

    vpc_id: str
    """
    VPC the Private Network belongs to.
    """


@dataclass
class SetSubnetsResponse:
    subnets: List[str]


@dataclass
class Subnet:
    """
    Subnet.
    """

    id: str
    """
    ID of the subnet.
    """

    created_at: Optional[datetime]
    """
    Subnet creation date.
    """

    updated_at: Optional[datetime]
    """
    Subnet last modification date.
    """

    subnet: str
    """
    Subnet CIDR.
    """


@dataclass
class VPC:
    """
    Vpc.
    """

    id: str
    """
    VPC ID.
    """

    name: str
    """
    VPC name.
    """

    organization_id: str
    """
    Scaleway Organization the VPC belongs to.
    """

    project_id: str
    """
    Scaleway Project the VPC belongs to.
    """

    region: Region
    """
    Region of the VPC.
    """

    tags: List[str]
    """
    Tags for the VPC.
    """

    is_default: bool
    """
    Defines whether the VPC is the default one for its Project.
    """

    created_at: Optional[datetime]
    """
    Date the VPC was created.
    """

    updated_at: Optional[datetime]
    """
    Date the VPC was last modified.
    """

    private_network_count: int
    """
    Number of Private Networks within this VPC.
    """


@dataclass
class ListVPCsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListVPCsRequestOrderBy]
    """
    Sort order of the returned VPCs.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of VPCs to return per page.
    """

    name: Optional[str]
    """
    Name to filter for. Only VPCs with names containing this string will be returned.
    """

    tags: Optional[List[str]]
    """
    Tags to filter for. Only VPCs with one more more matching tags will be returned.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for. Only VPCs belonging to this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for. Only VPCs belonging to this Project will be returned.
    """

    is_default: Optional[bool]
    """
    Defines whether to filter only for VPCs which are the default one for their Project.
    """


@dataclass
class CreateVPCRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name for the VPC.
    """

    project_id: Optional[str]
    """
    Scaleway Project in which to create the VPC.
    """

    tags: Optional[List[str]]
    """
    Tags for the VPC.
    """


@dataclass
class GetVPCRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    vpc_id: str
    """
    VPC ID.
    """


@dataclass
class UpdateVPCRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    vpc_id: str
    """
    VPC ID.
    """

    name: Optional[str]
    """
    Name for the VPC.
    """

    tags: Optional[List[str]]
    """
    Tags for the VPC.
    """


@dataclass
class DeleteVPCRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    vpc_id: str
    """
    VPC ID.
    """


@dataclass
class ListPrivateNetworksRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListPrivateNetworksRequestOrderBy]
    """
    Sort order of the returned Private Networks.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of Private Networks to return per page.
    """

    name: Optional[str]
    """
    Name to filter for. Only Private Networks with names containing this string will be returned.
    """

    tags: Optional[List[str]]
    """
    Tags to filter for. Only Private Networks with one or more matching tags will be returned.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for. Only Private Networks belonging to this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for. Only Private Networks belonging to this Project will be returned.
    """

    private_network_ids: Optional[List[str]]
    """
    Private Network IDs to filter for. Only Private Networks with one of these IDs will be returned.
    """

    vpc_id: Optional[str]
    """
    VPC ID to filter for. Only Private Networks belonging to this VPC will be returned.
    """


@dataclass
class CreatePrivateNetworkRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name for the Private Network.
    """

    project_id: Optional[str]
    """
    Scaleway Project in which to create the Private Network.
    """

    tags: Optional[List[str]]
    """
    Tags for the Private Network.
    """

    subnets: Optional[List[str]]
    """
    Private Network subnets CIDR.
    """

    vpc_id: Optional[str]
    """
    VPC in which to create the Private Network.
    """


@dataclass
class GetPrivateNetworkRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """


@dataclass
class UpdatePrivateNetworkRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    name: Optional[str]
    """
    Name for the Private Network.
    """

    tags: Optional[List[str]]
    """
    Tags for the Private Network.
    """


@dataclass
class DeletePrivateNetworkRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """


@dataclass
class MigrateZonalPrivateNetworksRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    organization_id: Optional[str]
    """
    Organization ID to target. The specified zoned Private Networks within this Organization will be migrated to regional.
    
    One-of ('scope'): at most one of 'organization_id', 'project_id' could be set.
    """

    project_id: Optional[str]
    """
    Project to target. The specified zoned Private Networks within this Project will be migrated to regional.
    
    One-of ('scope'): at most one of 'organization_id', 'project_id' could be set.
    """

    private_network_ids: Optional[List[str]]
    """
    IDs of the Private Networks to migrate.
    """


@dataclass
class SetSubnetsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    subnets: Optional[List[str]]
    """
    Private Network subnets CIDR.
    """


@dataclass
class AddSubnetsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    subnets: Optional[List[str]]
    """
    Private Network subnets CIDR.
    """


@dataclass
class DeleteSubnetsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    subnets: Optional[List[str]]
    """
    Private Network subnets CIDR.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ACLRuleAction
from .types import ACLRuleDirection
from .types import ACLRuleProtocol
from .types import DatabaseBackupStatus
from .types import EngineSettingPropertyType
from .types import InstanceLogStatus
from .types import InstanceStatus
from .types import ListDatabaseBackupsRequestOrderBy
from .types import ListDatabasesRequestOrderBy
from .types import ListInstanceLogsRequestOrderBy
from .types import ListInstancesRequestOrderBy
from .types import ListPrivilegesRequestOrderBy
from .types import ListSnapshotsRequestOrderBy
from .types import ListUsersRequestOrderBy
from .types import MaintenanceStatus
from .types import NodeTypeGeneration
from .types import NodeTypeStock
from .types import Permission
from .types import ReadReplicaStatus
from .types import SnapshotStatus
from .types import VolumeType
from .types import ACLRule
from .types import ACLRuleRequest
from .types import AddInstanceACLRulesResponse
from .types import AddInstanceSettingsResponse
from .types import BackupSchedule
from .types import Database
from .types import DatabaseBackup
from .types import DatabaseEngine
from .types import DeleteInstanceACLRulesResponse
from .types import DeleteInstanceSettingsResponse
from .types import Endpoint
from .types import EndpointDirectAccessDetails
from .types import EndpointLoadBalancerDetails
from .types import EndpointPrivateNetworkDetails
from .types import EndpointSpec
from .types import EndpointSpecLoadBalancer
from .types import EndpointSpecPrivateNetwork
from .types import EndpointSpecPrivateNetworkIpamConfig
from .types import EngineSetting
from .types import EngineVersion
from .types import Instance
from .types import InstanceLog
from .types import InstanceMetrics
from .types import InstanceSetting
from .types import ListDatabaseBackupsResponse
from .types import ListDatabaseEnginesResponse
from .types import ListDatabasesResponse
from .types import ListInstanceACLRulesResponse
from .types import ListInstanceLogsDetailsResponse
from .types import ListInstanceLogsDetailsResponseInstanceLogDetail
from .types import ListInstanceLogsResponse
from .types import ListInstancesResponse
from .types import ListNodeTypesResponse
from .types import ListPrivilegesResponse
from .types import ListSnapshotsResponse
from .types import ListUsersResponse
from .types import LogsPolicy
from .types import Maintenance
from .types import NodeType
from .types import NodeTypeVolumeConstraintSizes
from .types import NodeTypeVolumeType
from .types import PrepareInstanceLogsResponse
from .types import Privilege
from .types import ReadReplica
from .types import ReadReplicaEndpointSpec
from .types import ReadReplicaEndpointSpecDirectAccess
from .types import ReadReplicaEndpointSpecPrivateNetwork
from .types import ReadReplicaEndpointSpecPrivateNetworkIpamConfig
from .types import SetInstanceACLRulesResponse
from .types import SetInstanceSettingsResponse
from .types import Snapshot
from .types import UpgradableVersion
from .types import User
from .types import Volume
from .content import DATABASE_BACKUP_TRANSIENT_STATUSES
from .content import INSTANCE_LOG_TRANSIENT_STATUSES
from .content import INSTANCE_TRANSIENT_STATUSES
from .content import MAINTENANCE_TRANSIENT_STATUSES
from .content import READ_REPLICA_TRANSIENT_STATUSES
from .content import SNAPSHOT_TRANSIENT_STATUSES
from .api import RdbV1API

__all__ = [
    "ACLRuleAction",
    "ACLRuleDirection",
    "ACLRuleProtocol",
    "DatabaseBackupStatus",
    "EngineSettingPropertyType",
    "InstanceLogStatus",
    "InstanceStatus",
    "ListDatabaseBackupsRequestOrderBy",
    "ListDatabasesRequestOrderBy",
    "ListInstanceLogsRequestOrderBy",
    "ListInstancesRequestOrderBy",
    "ListPrivilegesRequestOrderBy",
    "ListSnapshotsRequestOrderBy",
    "ListUsersRequestOrderBy",
    "MaintenanceStatus",
    "NodeTypeGeneration",
    "NodeTypeStock",
    "Permission",
    "ReadReplicaStatus",
    "SnapshotStatus",
    "VolumeType",
    "ACLRule",
    "ACLRuleRequest",
    "AddInstanceACLRulesResponse",
    "AddInstanceSettingsResponse",
    "BackupSchedule",
    "Database",
    "DatabaseBackup",
    "DatabaseEngine",
    "DeleteInstanceACLRulesResponse",
    "DeleteInstanceSettingsResponse",
    "Endpoint",
    "EndpointDirectAccessDetails",
    "EndpointLoadBalancerDetails",
    "EndpointPrivateNetworkDetails",
    "EndpointSpec",
    "EndpointSpecLoadBalancer",
    "EndpointSpecPrivateNetwork",
    "EndpointSpecPrivateNetworkIpamConfig",
    "EngineSetting",
    "EngineVersion",
    "Instance",
    "InstanceLog",
    "InstanceMetrics",
    "InstanceSetting",
    "ListDatabaseBackupsResponse",
    "ListDatabaseEnginesResponse",
    "ListDatabasesResponse",
    "ListInstanceACLRulesResponse",
    "ListInstanceLogsDetailsResponse",
    "ListInstanceLogsDetailsResponseInstanceLogDetail",
    "ListInstanceLogsResponse",
    "ListInstancesResponse",
    "ListNodeTypesResponse",
    "ListPrivilegesResponse",
    "ListSnapshotsResponse",
    "ListUsersResponse",
    "LogsPolicy",
    "Maintenance",
    "NodeType",
    "NodeTypeVolumeConstraintSizes",
    "NodeTypeVolumeType",
    "PrepareInstanceLogsResponse",
    "Privilege",
    "ReadReplica",
    "ReadReplicaEndpointSpec",
    "ReadReplicaEndpointSpecDirectAccess",
    "ReadReplicaEndpointSpecPrivateNetwork",
    "ReadReplicaEndpointSpecPrivateNetworkIpamConfig",
    "SetInstanceACLRulesResponse",
    "SetInstanceSettingsResponse",
    "Snapshot",
    "UpgradableVersion",
    "User",
    "Volume",
    "DATABASE_BACKUP_TRANSIENT_STATUSES",
    "INSTANCE_LOG_TRANSIENT_STATUSES",
    "INSTANCE_TRANSIENT_STATUSES",
    "MAINTENANCE_TRANSIENT_STATUSES",
    "READ_REPLICA_TRANSIENT_STATUSES",
    "SNAPSHOT_TRANSIENT_STATUSES",
    "RdbV1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Zone,
)
from scaleway_core.utils import (
    OneOfPossibility,
    fetch_all_pages,
    resolve_one_of,
    validate_path_param,
)
from .types import (
    ListImagesRequestOrderBy,
    ListLocalImagesRequestOrderBy,
    ListVersionsRequestOrderBy,
    Category,
    Image,
    ListCategoriesResponse,
    ListImagesResponse,
    ListLocalImagesResponse,
    ListVersionsResponse,
    LocalImage,
    Version,
)
from .marshalling import (
    unmarshal_Category,
    unmarshal_Image,
    unmarshal_LocalImage,
    unmarshal_Version,
    unmarshal_ListCategoriesResponse,
    unmarshal_ListImagesResponse,
    unmarshal_ListLocalImagesResponse,
    unmarshal_ListVersionsResponse,
)


class MarketplaceV2API(API):
    """
    Marketplace API.

    Marketplace API.
    """

    def list_images(
        self,
        *,
        include_eol: bool,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
        order_by: ListImagesRequestOrderBy = ListImagesRequestOrderBy.NAME_ASC,
        arch: Optional[str] = None,
        category: Optional[str] = None,
    ) -> ListImagesResponse:
        """
        List marketplace images.
        List all available images on the marketplace, their UUID, CPU architecture and description.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param page: A positive integer to choose the page to display.
        :param order_by: Ordering to use.
        :param arch: Choose for which machine architecture to return images.
        :param category: Choose the category of images to get.
        :param include_eol: Choose to include end-of-life images.
        :return: :class:`ListImagesResponse <ListImagesResponse>`

        Usage:
        ::

            result = api.list_images(include_eol=True)
        """

        res = self._request(
            "GET",
            f"/marketplace/v2/images",
            params={
                "arch": arch,
                "category": category,
                "include_eol": include_eol,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListImagesResponse(res.json())

    def list_images_all(
        self,
        *,
        include_eol: bool,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
        order_by: Optional[ListImagesRequestOrderBy] = None,
        arch: Optional[str] = None,
        category: Optional[str] = None,
    ) -> List[Image]:
        """
        List marketplace images.
        List all available images on the marketplace, their UUID, CPU architecture and description.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param page: A positive integer to choose the page to display.
        :param order_by: Ordering to use.
        :param arch: Choose for which machine architecture to return images.
        :param category: Choose the category of images to get.
        :param include_eol: Choose to include end-of-life images.
        :return: :class:`List[ListImagesResponse] <List[ListImagesResponse]>`

        Usage:
        ::

            result = api.list_images_all(include_eol=True)
        """

        return fetch_all_pages(
            type=ListImagesResponse,
            key="images",
            fetcher=self.list_images,
            args={
                "include_eol": include_eol,
                "page_size": page_size,
                "page": page,
                "order_by": order_by,
                "arch": arch,
                "category": category,
            },
        )

    def get_image(
        self,
        *,
        image_id: str,
    ) -> Image:
        """
        Get a specific marketplace image.
        Get detailed information about a marketplace image, specified by its `image_id` (UUID format).
        :param image_id: Display the image name.
        :return: :class:`Image <Image>`

        Usage:
        ::

            result = api.get_image(image_id="example")
        """

        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "GET",
            f"/marketplace/v2/images/{param_image_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Image(res.json())

    def list_versions(
        self,
        *,
        image_id: str,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
        order_by: ListVersionsRequestOrderBy = ListVersionsRequestOrderBy.CREATED_AT_ASC,
    ) -> ListVersionsResponse:
        """
        List versions of an Image.
        Get a list of all available version of an image, specified by its `image_id` (UUID format).
        :param image_id:
        :param page_size:
        :param page:
        :param order_by:
        :return: :class:`ListVersionsResponse <ListVersionsResponse>`

        Usage:
        ::

            result = api.list_versions(image_id="example")
        """

        res = self._request(
            "GET",
            f"/marketplace/v2/versions",
            params={
                "image_id": image_id,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListVersionsResponse(res.json())

    def list_versions_all(
        self,
        *,
        image_id: str,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
        order_by: Optional[ListVersionsRequestOrderBy] = None,
    ) -> List[Version]:
        """
        List versions of an Image.
        Get a list of all available version of an image, specified by its `image_id` (UUID format).
        :param image_id:
        :param page_size:
        :param page:
        :param order_by:
        :return: :class:`List[ListVersionsResponse] <List[ListVersionsResponse]>`

        Usage:
        ::

            result = api.list_versions_all(image_id="example")
        """

        return fetch_all_pages(
            type=ListVersionsResponse,
            key="versions",
            fetcher=self.list_versions,
            args={
                "image_id": image_id,
                "page_size": page_size,
                "page": page,
                "order_by": order_by,
            },
        )

    def get_version(
        self,
        *,
        version_id: str,
    ) -> Version:
        """
        Get a specific image version.
        Get information such as the name, creation date, last update and published date for an image version specified by its `version_id` (UUID format).
        :param version_id:
        :return: :class:`Version <Version>`

        Usage:
        ::

            result = api.get_version(version_id="example")
        """

        param_version_id = validate_path_param("version_id", version_id)

        res = self._request(
            "GET",
            f"/marketplace/v2/versions/{param_version_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Version(res.json())

    def list_local_images(
        self,
        *,
        image_id: Optional[str] = None,
        version_id: Optional[str] = None,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
        order_by: ListLocalImagesRequestOrderBy = ListLocalImagesRequestOrderBy.CREATED_AT_ASC,
        image_label: Optional[str] = None,
        zone: Optional[Zone] = None,
    ) -> ListLocalImagesResponse:
        """
        List local images from a specific image or version.
        List information about local images in a specific Availability Zone, specified by its `image_id` (UUID format), `version_id` (UUID format) or `image_label`. Only one of these three parameters may be set.
        :param image_id: One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
        :param version_id: One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
        :param page_size:
        :param page:
        :param order_by:
        :param image_label: One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
        :param zone:
        :return: :class:`ListLocalImagesResponse <ListLocalImagesResponse>`

        Usage:
        ::

            result = api.list_local_images()
        """

        res = self._request(
            "GET",
            f"/marketplace/v2/local-images",
            params={
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "zone": zone or self.client.default_zone,
                **resolve_one_of(
                    [
                        OneOfPossibility("image_id", image_id),
                        OneOfPossibility("version_id", version_id),
                        OneOfPossibility("image_label", image_label),
                    ]
                ),
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListLocalImagesResponse(res.json())

    def list_local_images_all(
        self,
        *,
        image_id: Optional[str] = None,
        version_id: Optional[str] = None,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
        order_by: Optional[ListLocalImagesRequestOrderBy] = None,
        image_label: Optional[str] = None,
        zone: Optional[Zone] = None,
    ) -> List[LocalImage]:
        """
        List local images from a specific image or version.
        List information about local images in a specific Availability Zone, specified by its `image_id` (UUID format), `version_id` (UUID format) or `image_label`. Only one of these three parameters may be set.
        :param image_id: One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
        :param version_id: One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
        :param page_size:
        :param page:
        :param order_by:
        :param image_label: One-of ('scope'): at most one of 'image_id', 'version_id', 'image_label' could be set.
        :param zone:
        :return: :class:`List[ListLocalImagesResponse] <List[ListLocalImagesResponse]>`

        Usage:
        ::

            result = api.list_local_images_all()
        """

        return fetch_all_pages(
            type=ListLocalImagesResponse,
            key="local_images",
            fetcher=self.list_local_images,
            args={
                "image_id": image_id,
                "version_id": version_id,
                "page_size": page_size,
                "page": page,
                "order_by": order_by,
                "image_label": image_label,
                "zone": zone,
            },
        )

    def get_local_image(
        self,
        *,
        local_image_id: str,
    ) -> LocalImage:
        """
        Get a specific local image by ID.
        Get detailed information about a local image, including compatible commercial types, supported architecture, labels and the Availability Zone of the image, specified by its `local_image_id` (UUID format).
        :param local_image_id:
        :return: :class:`LocalImage <LocalImage>`

        Usage:
        ::

            result = api.get_local_image(local_image_id="example")
        """

        param_local_image_id = validate_path_param("local_image_id", local_image_id)

        res = self._request(
            "GET",
            f"/marketplace/v2/local-images/{param_local_image_id}",
        )

        self._throw_on_error(res)
        return unmarshal_LocalImage(res.json())

    def list_categories(
        self,
        *,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
    ) -> ListCategoriesResponse:
        """
        List existing image categories.
        Get a list of all existing categories. The output can be paginated.
        :param page_size:
        :param page:
        :return: :class:`ListCategoriesResponse <ListCategoriesResponse>`

        Usage:
        ::

            result = api.list_categories()
        """

        res = self._request(
            "GET",
            f"/marketplace/v2/categories",
            params={
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListCategoriesResponse(res.json())

    def list_categories_all(
        self,
        *,
        page_size: Optional[int] = None,
        page: Optional[int] = None,
    ) -> List[Category]:
        """
        List existing image categories.
        Get a list of all existing categories. The output can be paginated.
        :param page_size:
        :param page:
        :return: :class:`List[ListCategoriesResponse] <List[ListCategoriesResponse]>`

        Usage:
        ::

            result = api.list_categories_all()
        """

        return fetch_all_pages(
            type=ListCategoriesResponse,
            key="categories",
            fetcher=self.list_categories,
            args={
                "page_size": page_size,
                "page": page,
            },
        )

    def get_category(
        self,
        *,
        category_id: str,
    ) -> Category:
        """
        Get a specific category.
        Get information about a specific category of the marketplace catalog, specified by its `category_id` (UUID format).
        :param category_id:
        :return: :class:`Category <Category>`

        Usage:
        ::

            result = api.get_category(category_id="example")
        """

        param_category_id = validate_path_param("category_id", category_id)

        res = self._request(
            "GET",
            f"/marketplace/v2/categories/{param_category_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Category(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    NamespaceProtocol,
    Credential,
    CredentialNATSCredsFile,
    CredentialSQSSNSCreds,
    CredentialSummary,
    CredentialSummarySQSSNSCreds,
    ListCredentialsResponse,
    ListNamespacesResponse,
    Namespace,
    Permissions,
    CreateNamespaceRequest,
    UpdateNamespaceRequest,
    CreateCredentialRequest,
    UpdateCredentialRequest,
)


def unmarshal_Permissions(data: Any) -> Permissions:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Permissions' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("can_manage")
    args["can_manage"] = field

    field = data.get("can_publish")
    args["can_publish"] = field

    field = data.get("can_receive")
    args["can_receive"] = field

    return Permissions(**args)


def unmarshal_CredentialSummarySQSSNSCreds(data: Any) -> CredentialSummarySQSSNSCreds:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CredentialSummarySQSSNSCreds' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("access_key")
    args["access_key"] = field

    field = data.get("permissions")
    args["permissions"] = unmarshal_Permissions(field) if field is not None else None

    return CredentialSummarySQSSNSCreds(**args)


def unmarshal_CredentialNATSCredsFile(data: Any) -> CredentialNATSCredsFile:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CredentialNATSCredsFile' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("content")
    args["content"] = field

    return CredentialNATSCredsFile(**args)


def unmarshal_CredentialSQSSNSCreds(data: Any) -> CredentialSQSSNSCreds:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CredentialSQSSNSCreds' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("access_key")
    args["access_key"] = field

    field = data.get("permissions")
    args["permissions"] = unmarshal_Permissions(field) if field is not None else None

    field = data.get("secret_key")
    args["secret_key"] = field

    return CredentialSQSSNSCreds(**args)


def unmarshal_CredentialSummary(data: Any) -> CredentialSummary:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CredentialSummary' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("namespace_id")
    args["namespace_id"] = field

    field = data.get("protocol")
    args["protocol"] = field

    field = data.get("sqs_sns_credentials")
    args["sqs_sns_credentials"] = (
        unmarshal_CredentialSummarySQSSNSCreds(field) if field is not None else None
    )

    return CredentialSummary(**args)


def unmarshal_Namespace(data: Any) -> Namespace:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Namespace' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("endpoint")
    args["endpoint"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("protocol")
    args["protocol"] = field

    field = data.get("region")
    args["region"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Namespace(**args)


def unmarshal_Credential(data: Any) -> Credential:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Credential' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("namespace_id")
    args["namespace_id"] = field

    field = data.get("nats_credentials")
    args["nats_credentials"] = (
        unmarshal_CredentialNATSCredsFile(field) if field is not None else None
    )

    field = data.get("protocol")
    args["protocol"] = field

    field = data.get("sqs_sns_credentials")
    args["sqs_sns_credentials"] = (
        unmarshal_CredentialSQSSNSCreds(field) if field is not None else None
    )

    return Credential(**args)


def unmarshal_ListCredentialsResponse(data: Any) -> ListCredentialsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListCredentialsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("credentials")
    args["credentials"] = [unmarshal_CredentialSummary(v) for v in data["credentials"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListCredentialsResponse(**args)


def unmarshal_ListNamespacesResponse(data: Any) -> ListNamespacesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListNamespacesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("namespaces")
    args["namespaces"] = [unmarshal_Namespace(v) for v in data["namespaces"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListNamespacesResponse(**args)


def marshal_Permissions(
    request: Permissions,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "can_manage": request.can_manage,
        "can_publish": request.can_publish,
        "can_receive": request.can_receive,
    }


def marshal_CreateCredentialRequest(
    request: CreateCredentialRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "name": request.name,
        "namespace_id": request.namespace_id,
        "permissions": marshal_Permissions(request.permissions, defaults)
        if request.permissions is not None
        else None,
    }


def marshal_CreateNamespaceRequest(
    request: CreateNamespaceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "name": request.name,
        "project_id": request.project_id or defaults.default_project_id,
        "protocol": NamespaceProtocol(request.protocol)
        if request.protocol is not None
        else None,
    }


def marshal_UpdateCredentialRequest(
    request: UpdateCredentialRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "name": request.name,
        "permissions": marshal_Permissions(request.permissions, defaults)
        if request.permissions is not None
        else None,
    }


def marshal_UpdateNamespaceRequest(
    request: UpdateNamespaceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "name": request.name,
        "namespace_id": request.namespace_id,
    }
